<?php

namespace Myriad\NotifyClient\Builders;

use Exception;
use const DEBUG_BACKTRACE_IGNORE_ARGS;

class FilterBuilder
{

    private $channels = null;

    private $raw = null;

    private $serviceTypes = null;

    private $shouldValidate = true;

    private $subscriptions = null;

    private $uids = null;

    private $variants = null;

    /**
     * @param $val
     * @return $this
     * @throws Exception
     */
    public function onChannels($val): self
    {
        $this->assertNotRaw();
        $this->channels = array_filter(is_array($val) ? $val : func_get_args());
        return $this;
    }

    /**
     * @param $val
     * @return $this
     * @throws Exception
     */
    public function forSubscriptions($val): self
    {
        $this->assertNotRaw();
        $this->subscriptions = array_filter(is_array($val) ? $val : func_get_args());
        return $this;
    }

    /**
     * @param $val
     * @return $this
     * @throws Exception
     */
    public function toUids($val): self
    {
        $this->assertNotRaw();
        $this->uids = array_filter(is_array($val) ? $val : func_get_args());
        return $this;
    }

    /**
     * @param $val
     * @return $this
     * @throws Exception
     */
    public function viaServices($val): self
    {
        $this->assertNotRaw();
        $this->serviceTypes = array_filter(is_array($val) ? $val : func_get_args());
        return $this;
    }

    /**
     * @param $val
     * @return $this
     * @throws Exception
     */
    public function toVariants($val): self
    {
        $this->assertNotRaw();
        $this->variants = array_filter(is_array($val) ? $val : func_get_args());
        return $this;
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function withoutValidation(): self
    {
        $this->assertNotRaw();
        $this->shouldValidate = false;
        return $this;
    }

    /**
     * @return array|null
     */
    public function build(): ?array
    {
        if ($this->isRaw()) {
            return $this->raw;
        }

        $filter = [];
        if ($this->channels !== null) {
            $filter['channels'] = ['$in' => $this->channels];
        }
        if ($this->serviceTypes !== null) {
            $filter['registrants.serviceType'] = ['$in' => $this->serviceTypes];
        }
        if ($this->subscriptions !== null) {
            $filter['registrants.subscriptions'] = ['$in' => $this->subscriptions];
        }
        if ($this->uids !== null) {
            $filter['uid'] = ['$in' => $this->uids];
        }
        if ($this->variants !== null) {
            $filter['registrants.variant'] = ['$in' => $this->variants];
        }

        return $filter;
    }

    /**
     * @return bool
     */
    public function isRaw(): bool
    {
        return !is_null($this->raw);
    }

    /**
     * @return self
     */
    public static function create(): self
    {
        return new self;
    }

    /**
     * @param  array  $filter
     * @return self
     */
    public static function raw(array $filter): self
    {
        $filterObj = new self;
        $filterObj->configureRaw($filter);
        return $filterObj;
    }

    /**
     * @param $filter
     */
    private function configureRaw($filter): void
    {
        $this->shouldValidate = false;
        $this->raw = $filter;
    }

    /**
     * @throws Exception
     */
    private function assertNotRaw(): void
    {
        if ($this->isRaw()) {
            $trace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2);
            $methodName = $trace[1]['function'];
            throw new Exception("Cannot use `$methodName` when the FilterBuilder has been created using `FilterBuilder::raw`");
        }
    }

    /**
     * @param $name
     * @return mixed
     */
    function __get($name)
    {
        return $this->{$name};
    }
}
