<?php namespace Myriad\NotifyClient\Client;

class Config
{

    /**
     * The API token
     * @var string
     */
    private $token;

    /**
     * The list of channels supported by the application.
     * @var mixed
     */
    private $channels;

    /**
     * The list of subscriptions supported by the application.
     * @var mixed
     */
    private $subscriptions;

    /**
     * When true, before the request is made, the requested channels and subscriptions are
     * checked against the lists configured for the client (by default, the values in
     * config/notification.php are used). If a requested channel or subscription is not found,
     * an exception will be thrown.
     * @var bool
     */
    private $shouldVerify;

    /**
     * @var string
     */
    private $baseUrl;

    function __construct(
        $token,
        $shouldVerify = true,
        $channels = [],
        $subscriptions = [],
        $baseUrl = 'https://notify.myriadapps.com/api/v1/'
    ) {
        $this->channels = $channels;
        $this->shouldVerify = $shouldVerify;
        $this->subscriptions = $subscriptions;
        $this->token = $token;
        $this->baseUrl = $baseUrl.(substr($baseUrl, -1) === '/' ? '' : '/');
    }

    /**
     * @return array
     */
    public function getChannels(): array
    {
        return $this->channels;
    }

    /**
     * @return array
     */
    public function getSubscriptions(): array
    {
        return $this->subscriptions;
    }

    /**
     * @return bool
     */
    public function shouldVerify(): bool
    {
        return $this->shouldVerify;
    }

    /**
     * @return string
     */
    public function getToken(): string
    {
        return $this->token;
    }

    /**
     * @return string
     */
    public function getBaseUrl(): string
    {
        return $this->baseUrl;
    }
}
