<?php

namespace Myriad\NotifyClient\Client\FluentApi;

use InvalidArgumentException;
use Myriad\NotifyClient\Client;

class NotificationGroupFluentApi
{

    private $nsc;

    private $uid;

    public function __construct(Client $nsc, $uid = null)
    {
        $this->nsc = $nsc;
        $this->uid = $uid;
    }

    // $nsc->notificationGroups($uid)->createUpdate([ 'timezone' => 'America/Chicago', ... ]);
    // $nsc->notificationGroups($uid)->createUpdate([ 'latitude' => 11.321, ... ]);
    // $nsc->notificationGroups()->createUpdate([ 'uid' => '8234', 'latitude' => 11.321, ... ]);

    /**
     * @param $notificationGroup
     * @return mixed
     */
    public function createUpdate($notificationGroup)
    {
        return $this->nsc->createUpdateNotificationGroup($notificationGroup, $this->uid);
    }

    // $nsc->notificationGroups()->query($filter);

    /**
     * @param $filter
     * @param  null  $sort
     * @param  null  $limit
     * @return array
     */
    public function query($filter, $sort = null, $limit = null): array
    {
        return $this->nsc->queryNotificationGroups($filter, $sort, $limit);
    }

    // $nsc->notificationGroups($uid)->get();

    /**
     * @return mixed
     */
    public function get()
    {
        return $this->nsc->getNotificationGroup($this->uid);
    }

    // $nsc->notificationGroups($uid)->delete();

    /**
     * @return mixed
     */
    public function delete()
    {
        return $this->nsc->deleteNotificationGroup($this->uid);
    }

    // $nsc->notificationGroups($uid)->addChannels('channel', 'other');

    /**
     * @param $val
     * @return mixed
     */
    public function addChannels($val)
    {
        $channels = is_array($val) ? $val : func_get_args();

        return $this->nsc->addChannelsToNotificationGroup($this->uid, $channels);
    }

    // $nsc->notificationGroups($uid)->removeChannels('channel', 'other');

    /**
     * @param $val
     * @return mixed
     */
    public function removeChannels($val)
    {
        $channels = is_array($val) ? $val : func_get_args();

        return $this->nsc->removeChannelsFromNotificationGroup($this->uid, $channels);
    }

    // $nsc->notificationGroups($uid)->addUpdateCustom(['key' => 'value', ...]);

    /**
     * @param $arr
     * @return mixed
     */
    public function addUpdateCustom($arr)
    {
        return $this->nsc->addUpdateCustomToNotificationGroup($this->uid, $arr);
    }

    // $nsc->notificationGroups($uid)->removeCustom(['key', 'key2']);

    /**
     * @param $val
     * @return mixed
     */
    public function removeCustom($val)
    {
        $custom = is_array($val) ? $val : func_get_args();
        return $this->nsc->removeCustomFromNotificationGroup($this->uid, $custom);
    }

    //public function updateTelemetry ($arr) {}

    // $nsc->notificationGroups($uid)->registrant...
    // $nsc->notificationGroups()->registrant... // throws

    /**
     * @param  null  $registrantSlug
     * @return NotificationGroupRegistrantFluentApi
     * @throws InvalidArgumentException
     */
    public function registrant($registrantSlug = null): NotificationGroupRegistrantFluentApi
    {
        if (!$this->uid) {
            throw new InvalidArgumentException(
                'A uid must be provided to `notificationGroup` in order to access registrant methods.'
            );
        }
        return new NotificationGroupRegistrantFluentApi($this->nsc, $this->uid, $registrantSlug);
    }
}
