<?php

namespace Myriad\NotifyClient\Client\Helpers;

use Exception;
use Myriad\NotifyClient\Builders\NotificationBuilder;
use Myriad\NotifyClient\Client\ValidatesAgainstConfig;
use Myriad\NotifyClient\Models\Notification;
use Myriad\NotifyClient\Utilities\BuiltWithConfig;
use Myriad\NotifyClient\Utilities\HydratesModels;

class NotificationHelper
{

    use BuiltWithConfig;
    use HydratesModels;
    use ValidatesAgainstConfig;

    /**
     * @return string
     */
    protected function getItemClass(): string
    {
        return Notification::class;
    }

    /**
     * @param $object
     * @return array
     * @throws Exception
     */
    public function makeArray($object): array
    {
        if ($object instanceof NotificationBuilder) {
            return $object->build()->toArray();
        }
        if ($object instanceof Notification) {
            return $object->toArray();
        }
        throw new Exception('Object must be a Notification, or NotificationBuilder');
    }

    /**
     * @param $notification
     * @throws Exception
     */
    public function maybeValidateBuilder($notification): void
    {
        if ($notification instanceof NotificationBuilder) {
            $filterBuilder = $notification->filterBuilder;
            if ($filterBuilder->shouldValidate) {
                if (empty($filterBuilder->build())) {
                    throw new Exception("The filter has no constraints. This means that *all* registered users would
          receive this message -- you probably don't want that. If you do, you can use the 'withoutValidation'
          method on the FilterBuilder to disable this check, or you can use FilterBuilder::raw and construct
          your own mongo filter. ");
                }
                $this->maybeAssertChannelsAreValid($filterBuilder->channels ?: []);
                $this->maybeAssertSubscriptionsAreValid($filterBuilder->subscriptions ?: []);
            }
        }
    }

    /**
     * @param  null  $notificationId
     * @return string
     */
    public function uri($notificationId = null): string
    {
        return "notifications".($notificationId ? "/$notificationId" : '');
    }
}
