<?php

namespace Myriad\NotifyClient\Client;

use Exception;
use Myriad\NotifyClient\HttpInterface;

/**
 * Class HttpWrapper
 * This class serves as a facade for interacting with the uri builder and the http client
 */
class HttpWrapper
{
    /**
     * The HTTP client.
     */
    private $http;

    /**
     * @var Config
     */
    private $config;

    function __construct(Config $config, HttpInterface $http)
    {
        $this->config = $config;
        $this->http = $http;
    }

    /**
     * @param $verb
     * @param $arguments
     * @return mixed
     * @throws Exception
     */
    private function makeRequest($verb, $arguments)
    {
        $arg1 = data_get($arguments, 0);
        $arg2 = data_get($arguments, 1);

        $url = $this->config->getBaseUrl();
        if (is_string($arg1)) {
            $url .= $arg1;
            $json = $arg2;
        } else {
            $json = $arg1;
        }

        $headers = [
            'Accept' => 'application/json',
            'Content-Type' => 'application/json',
            'Notify-Authorization' => 'Notify-Server '.$this->config->getToken(),
        ];

        $response = $this->http->request($verb, $url, $headers, $json);

        if (is_array($response) && array_key_exists('data', $response)) {
            return $response['data'];
        } else {
            return $response;
        }
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function get()
    {
        return $this->makeRequest('get', func_get_args());
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function put()
    {
        return $this->makeRequest('put', func_get_args());
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function post()
    {
        return $this->makeRequest('post', func_get_args());
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function delete()
    {
        return $this->makeRequest('delete', func_get_args());
    }
}
